#
# See the file "LICENSE" for the full license governing this code.
#
#++--++
#
# iib_install::autostart
#
# @summary This defined type makes sure the specfied IIB broker is automaticaly started at system start.
# Here is an example on how to use it:
# 
# ``` puppet
# iib_install::autostart { 'IIBBROKER1':
#   ensure       => 'present',
#   path         => '/etc/systemd/system',
#   service_name => 'iib@',
#   install_root => '/opt/IBM',
#   iib_version  => '10.0.0.9',
#   iib_user     => 'iibadmin',
#   iib_group    => 'mqbrkrs',
# }
# ```
#
# @param [Enum['present', 'absent']] ensure
#    Tells if you want to make sure the broker is either there or not.
#    This property has twho possible values `present`, which makes sure the broker autostart is present, or `absent` which makes sure there is no autostart beeing applied to this broker.
#    Here is an example on how to use it:
#    ```puppet
#    iib_install::autostart { 'IIBBROKER1':
#      ...
#      ensure       => 'present',
#      ...
#    }
#    ```
#
# @param [String] path
#    Allows you to specify the path where the unit-file will be placed.
#    Here is an example on how to use this:
#    ``` puppet
#    iib_install::autostart{'IIBBROKER1':
#      ...
#      path         => '/etc/mysystemd/system',
#      ...
#    }
#    ```
#    The default value is `/etc/systemd/system'.
#
# @param [String] service_name
#    Allows you to specfy the name of the service.
#    Here is an example on how to use this:
#    ``` puppet
#    iib_install::autostart{'IIBBROKER1':
#      ...
#      service_name => 'my_iib,
#      ...
#    }
#    ```
#    The default value is : `"iib@$broker_name`.
#
# @param [String] install_root
#    Specfies the base path where the software used for the broker, is located,
#    Together with the `iib_version` parameter, this makes the full-path.
#    ``` puppet
#    iib_install::autostart { 'IIBBROKER1':
#      ...
#      install_root => '/opt/IBM',
#      ...
#    }
#    ```
#
# @param [String] iib_version
#    Specifies the version of IIB that is used for this autostart.
#    This parameter is used to construct the path of the software. 
#    Here is an example on how to use it:
#    ``` puppet
#    iib_install::autostart { 'IIBBROKER1':
#      ...
#      iib_version  => '10.0.0.9',
#      ...
#    }
#    ```
#
# @param [String] iib_user
#    Allows you to specify the OS user the IIB broker will be started under.
#    Here is an example on how to use this:
#    ``` puppet
#    iib_install::autostart{'IIBBROKER1':
#      ...
#      iib_user      => 'myuser',
#      ...
#    }
#    ```
#    The default value is `miibadmin` and is correct in most situations.
#
# @param [String] iib_group
#    Allows you to specify the OS group the IIB broker will be started under.
#    Here is an example on how to use this:
#    ``` puppet
#    iib_install::autostart{'BROKER1':
#      ...
#      iib_group     => 'mygroup',
#      ...
#    }
#    ```
#    The default value is `mqbrkrs` and is correct in most situations.
#
#--++--
define iib_install::autostart(
  Enum['present', 'absent'] $ensure = 'present',
  String  $path                     = lookup('iib_install::autostart::path'),
  String  $service_name             = lookup('iib_install::autostart::service_name'),
  String  $install_root             = lookup('iib_install::autostart::install_root'),
  String  $iib_version              = lookup('iib_install::autostart::iib_version'),
  String  $iib_user                 = lookup('iib_install::autostart::iib_user'),
  String  $iib_group                = lookup('iib_install::autostart::iib_group'),
)
{
  iib_install { "iib_install::autostart::${title}": }
  if $::osfamily == 'windows' { fail 'This module does not support Windows'}
  if $::systemd == undef  { fail 'Module camptocamp-systemd required. Probably not installed'}

  assert_type(Pattern[/\d+\.\d+\.\d+\.\d/], $iib_version) |$expected, $actual| {
    fail 'Invalid version specified'
  }

  case($ensure) {
    'absent':  {
      $service_ensure =  'stopped'
      $service_enable =  false
    }
    'present': {
      $service_ensure = 'running'
      $service_enable =  true
    }
    default: { fail('Invalid value for ensure found.')}
  }

  $iib_node_name = $title

  file {'/usr/local/bin/iibcmd':
    ensure  => 'present',
    content => epp('iib_install/iibcmd.epp', {
      'install_root' => $install_root,
      'iib_version'  => $iib_version,
      }),
    mode    => '0755',
  }

  if $::systemd {
    ::systemd::unit_file { "${service_name}.service":
      ensure  => $ensure,
      path    => $path,
      content => epp('iib_install/iib.service.epp', {
        'iib_version' => $iib_version,
        'iib_user'    => $iib_user,
        'iib_group'   => $iib_group,
        }),
      notify  => Service["${service_name}${iib_node_name}"],
    }

    # #
    # # When iib_broker starts the broker, it does so outside of the systemd context. To make
    # # sure we can start it with systemd, we first have to stop it and then start the service.
    # #
    -> exec {"stop ${iib_node_name} running outside systemd":
      command => "/usr/local/bin/iibcmd mqsistop ${iib_node_name}",
      path    => '/bin',
      user    => $iib_user,
      group   => $iib_group,
      onlyif  => "ps -eaf | grep 'bipbroker [${iib_node_name}]'",
      unless  => "systemctl --state running | grep ${iib_node_name}",
    }

    service {"${service_name}${iib_node_name}":
      ensure => $service_ensure,
      enable => $service_enable,
    }
  } else {

    file { "${path}/iib":
      ensure  => $ensure,
      content => epp('iib_install/iib.epp', {
        'install_root' => $install_root,
        'iib_version'  => $iib_version,
        'iib_user'     => $iib_user,
        }),
      mode    => '0755',
    }

    ~>service {$service_name:
      ensure => $service_ensure,
      enable => $service_enable,
    }
  }
}
