#
# See the file "LICENSE" for the full license governing this code.
#
#++--++
#
# iib_install::installiib
#
# @summary The defined type `iib_install::installiib` is a convenience type that allows you to install a specific version of the IBM Information Bus (IIB) software on your system.
# This defined type has support for installing multiple versions of the IIB software. This feature can help you seamlessly migrate your software from one version of IIB to an other one.
#
# Here is an example on how to use it:
#
# ``` puppet
# iib_install::installiib {...:
#   source_location   => '/software',
#   version           => '10.0.0.8',
#   iibadmin_password => 'verysecret',
# }
# ```
#
# @param [String] source_location
#    The location of the IIB software.
#    Here is an example on how to use this:
#    ```puppet
#    iib_install::... {...:
#      ...
#      source_location => '/software',
#      ...
#    }
#    ```
#
# @param [String] version
#    The version of IIB you want to install.
#    Here is an example on how to use it:
#    ``` puppet
#    iib_install::installiib {...:
#      ...
#      version           => '10.0.0.8',
#      ...
#    }
#    ```
#
# @param [String] iibadmin_password
#    The password to be used for the `iibadmin` user.
#    The default is `iibadmin`.
#    Here is an example on how to use it:
#    ``` puppet
#    iib_install::installiib {...:
#      ...
#      iibadmin_password => 'verysecret',
#      ...
#    }
#    ```
#
#--++--
define iib_install::installiib(
  String[1] $source_location,
  String[1] $version,
  Optional[String[1]]
            $file_name         = undef,
  Easy_type::Password $iibadmin_password = lookup({name => 'iib_install::installiib::iibadmin_password'})
)
{
  iib_install { "iib_install::installiib::${title}":}

  iib_install::os {'iibadmin':
    password => $iibadmin_password.unwrap,
  }

  -> iib_install::software { $version:
    source_location => $source_location,
    file_name       => $file_name,
  }
}
