#
# See the file "LICENSE" for the full license governing this code.
#
#++--++
#
# iib_install::os
#
# @summary The `iib_install::os` defined type is a convenience type for adding IIB os users to the system.
# Using this defined type you can add multiple users to the system.
# 
# Here is an example on how to use it:
# 
# ``` puppet
# iib_install::os {'my_iibadmin':
#   password => 'verysecret',
# }
# ```
#
# @param title
#    The OS user to add to the system and grant access to the MQ groups.
#    ```puppet
#    iib_install::os {'iibadmin':
#      ...
#    }
#    ```
#
# @param [String] password
#    The password to be used for user.
#    The default is the same as the username.
#    Here is an example on how to use this:
#    ```puppet
#    iib_install::os {...:
#      ...
#      password => 'verysecret',
#      ...
#    }
#    ```
#
#--++--
define iib_install::os(
  String $password = $title,
)
{
  iib_install { "iib_install::os::${title}":}

  unless defined(Group['mqbrkrs']) {
    group {'mqbrkrs':
      ensure => 'present',
    }
  }

  unless defined(Group['mqm']) {
    group {'mqm':
      ensure => 'present',
    }
  }

  user{ $title:
    ensure     => 'present',
    password   => pw_hash($password, 'SHA-512', regsubst($::networking[mac],':','','G')),
    home       => "/home/${title}",
    shell      => '/bin/bash',
    comment    => 'This user was created by Puppet',
    groups     => ['mqbrkrs','mqm'],
    managehome => true,
  }

}
