#
# See the file "LICENSE" for the full license governing this code.
#
#++--++
#
# iib_install::software
#
# @summary This defined type install's specified version of the the IIB software in the specified folder.
# The source zip file from IBM is found at the location specified at `source_location`.
#
# Here is an example on how to use it:
#
# ``` puppet
# iib_install::software{ '10.0.0.8':
#   source_location  => '/software',
#   install_root     => '/opt/IBM',
#   tmp_dir          => '/mytemp',
#   logoutput        => true,
# }
#
# ```
#
# @param [String] source_location
#    The location of the IIB software.
#    Here is an example on how to use this:
#    ```puppet
#    iib_install::... {...:
#      ...
#      source_location => '/software',
#      ...
#    }
#    ```
#
# @param [String] install_root
#    The location on the file system where the software will be places.
#    The default is '/opt/IBM'.
#    Here is an example on how to use this:
#    ```puppet
#    iib_install::software {...:
#      ...
#      install_root => /my_opt/IBM',
#      ...
#    }
#    ```
#
# @param [String] tmp_dir
#    This defined type uses a temporary directory.
#    By default this is `\tmp`. If you want to use an other directory for this, you must specify this parameter.
#    Here is an example on how to use this:
#    ```puppet
#    iib_install::software {...:
#      ...
#      tmp_dir => /my_tmp',
#      ...
#    }
#    ```
#
# @param [Variant[Enum['on_failure'], Boolean]] logoutput
#    If you want to see the output of the `exec` resources in the type, you can set this value to `true`.
#    The default is `on_failure`.
#    Here is an example on how to use this:
#    ```puppet
#    iib_install::... {...:
#      ...
#      logoutput => true,
#      ...
#    }
#    ```
#
#--++--
define iib_install::software(
  String[1]                            $source_location,
  Optional[String[1]]                  $file_name       = undef,
  Stdlib::Absolutepath                 $install_root    = '/opt/IBM',      # Root file where installation is places
  Stdlib::Absolutepath                 $tmp_dir         = '/tmp',          # Directory used for unpacking
  Variant[Enum['on_failure'], Boolean] $logoutput       = 'on_failure',    # Output of the execs
)
{
  if $::osfamily == 'windows' { fail 'This module does not support Windows'}
  iib_install { "iib_install::software::${title}":}

  $version = $title
  assert_type(Pattern[/\d+\.\d+\.\d+\.\d/], $version) |$expected, $actual| {
    fail 'Invalid version specified'
  }

  assert_type(Pattern[/\/opt\/.*/], $install_root) |$expected, $actual| {
    fail "install_root needs to be a sub directory of /opt, right now it isn't"
  }

  #
  $version_components = $version.split('\.')
  #
  # On the package version the last number is separated by an dash
  #
  $base_version    = "${version_components[0]}.${version_components[1]}.${version_components[2]}"
  $major_version   = $version_components[0]

  #
  # This only works on higher versions of Puppet
  # TODO: Use this when support for Puppet 4.3 and lower is removed.
  # $patch_level     = Integer($version_components[3])
  #
  $patch_level       = Integer($version_components[3])

  $supported_major_versions = ['10', '11']    # At this point in time only version 10 is tested. Probably other versions work as well.

  unless $major_version in $supported_major_versions {
    fail "IIB major version ${major_version} not (yet) supported. Only versions ${supported_major_versions.join(',')} supported."
  }

  case $major_version {
    '10': {
      $supported_patch_levels = range(6,14)
      if $file_name == undef {
        $final_file_name = "${base_version}-IIB-LINUXX64-FP${sprintf('%04d', $patch_level)}"
      } else {
        $final_file_name = $file_name
      }
      $product_name           = 'iib'
    }
    '11': {
      $supported_patch_levels   = range(0,4) # Support fixpack up to version 4
      if $file_name == undef {
        $final_file_name = "ace-${version}"
      } else {
        $final_file_name = $file_name
      }
      $product_name             = 'ace'
    }
    default: { fail 'internal error. Unkown version.'}
  }

  $dirs            = dirtree($install_root, '/opt')

  unless $patch_level in $supported_patch_levels {
    fail "IIB patch level ${patch_level} on major version ${major_version} not (yet) supported. Only patchlevels ${supported_patch_levels.join(',')} supported."
  }

  #
  # If the installed version is the same as the requested version. Do nothing
  #
  if $::iib_versions == undef or $version != $::iib_versions["${install_root}/${product_name}-${version}"] {

    $archive_title = "${install_root}/${final_file_name}.tar.gz"
    if $major_version == '10' and $patch_level < 8 {
      $software_file = "${tmp_dir}/EAsmbl_image/iib-${version}.tar.gz"
      iib_install::software::iib6_unpack{ $final_file_name:
        tmp_dir         => $tmp_dir,
        source_location => $source_location,
        before          => Archive[$archive_title],
      }
    } else {
      $software_file = "${source_location}/${final_file_name}.tar.gz"
    }

    $dirs.each |$dir| {
      unless defined(File[$dir]) {
        file {$dir:
          ensure => 'directory',
          owner  => 'root',
          mode   => '0644',
          before => Archive[$archive_title],
        }
      }
    }

    archive { $archive_title:
      ensure       => present,
      extract      => true,
      extract_path => $install_root,
      source       => $software_file,
      creates      => "${install_root}/${product_name}-${version}",
      cleanup      => true,
    }

    ~> exec { "Accept ${product_name} License Requirements for version ${version}":
      command     => "${install_root}/${product_name}-${version}/${product_name} make registry global accept license silently",
      refreshonly => true,
      logoutput   => $logoutput,
    }

    -> exec { "Install ${product_name} version ${version}":
      cwd       => "${install_root}/${product_name}-${version}",
      command   => "${install_root}/${product_name}-${version}/${product_name} verify all",
      logoutput => $logoutput,
    }
  }
}
